use Getopt::Long;
use Test::More;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw( GetError ExistsError Error );
use DE_EPAGES::Database::API::Connection qw( RunOnStore  );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Object::API::Factory qw(LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::GLS::API::RemoteGLSTransaction;
use DE_EPAGES::GLS::API::RemoteGLS;

use DE_EPAGES::GLS::t::MockupResponses;

use strict;

my $ROOTPATH = '/Test';


sub run_tests {

    RunOnStore(
        'Store' => 'Store',
        'Sub'   => sub {
            # initializsation
            ok( Init(), 'initialization' );
            eval {
                my ($continue, $hObjects) = testImport();
                return unless ( $continue );

                $hObjects->{'remoteTransaction'} =
                    DE_EPAGES::GLS::API::RemoteGLSTransaction->new($hObjects->{'shipFM'});

                my $aShippingLabels = getShippingLabels($hObjects);
                return unless ( $aShippingLabels );

                testGLSServices($hObjects, $aShippingLabels);

            };
            if (ExistsError()) {
                my $Error = GetError();
                fail('Error - '. $Error->code);
                diag($Error->longMessage);
                # always cleanup
                ok( Cleanup(), 'cleanup');
                Error($Error);
            }
            # cleanup
            ok( Cleanup(), 'cleanup');
        }
    );
}

sub testImport{
    my $testImportOK = 0;
    my $Shop = LoadObjectByPath($ROOTPATH)->child('TestShopGLS');
    $testImportOK = isa_ok($Shop, 'DE_EPAGES::Shop::API::Object::Shop');
    my $ShippingFulfillmentMethod = $Shop->path('ShippingFulfillmentMethods/GLSParcel');
    $testImportOK = isa_ok($ShippingFulfillmentMethod, 'DE_EPAGES::GLS::API::Object::ShippingFulfillmentMethodGLSParcel')
                        && $testImportOK;
    my $Customer = $Shop->path('Customers/TestCustomer');
    $testImportOK = isa_ok($Customer, 'DE_EPAGES::Customer::API::Object::Customer' ) && $testImportOK;
    my $Order = $Shop->path('Customers/TestCustomer/Orders/1234-5678');
    $testImportOK = isa_ok($Order, 'DE_EPAGES::Order::API::Object::CustomerOrder' ) && $testImportOK;
    $testImportOK = cmp_ok(scalar @{$Order->get('Documents')}, '>=',1, 'number of orderdocuments') && $testImportOK;
    my $ShippingLabel = $Shop->path('Customers/TestCustomer/Orders/1234-5678/ShippingLabels/1234-5678-1');
    $testImportOK = isa_ok($ShippingLabel, 'DE_EPAGES::GLS::API::Object::ShippingLabelGLS' ) && $testImportOK;
    my $Transaction = $Shop->path('ShippingOrders/ShippingOrderGLS');
    $testImportOK = isa_ok($Transaction, 'DE_EPAGES::ExternalShipping::API::Object::ShippingTransaction' )
                        && $testImportOK;
    $testImportOK = cmp_ok(scalar @{$Transaction->get('ShippingLabels')}, '>=',1, 'number of labels') && $testImportOK;

    foreach my $ShippingLabel (@{$Transaction->get('ShippingLabels')}) {
        $testImportOK = isa_ok($ShippingLabel, 'DE_EPAGES::GLS::API::Object::ShippingLabelGLS' ) && $testImportOK;
    }
    return ($testImportOK, {    'shop'          => $Shop,
                                'shipFM'        => $ShippingFulfillmentMethod,
                                'customer'      => $Customer,
                                'order'         => $Order,
                                'transaction'   => $Transaction,
    });
}

sub getShippingLabels{
    my ($hObjects) = @_;

    my $Transaction = $hObjects->{'transaction'};

    my $ShippingLabels = $Transaction->get('ShippingLabels');

    return $ShippingLabels;
}

sub testGLSServices{
    my ($hObjects, $aShippingLabels) = @_;

    my $RemoteTransaction = $hObjects->{'remoteTransaction'};

    foreach my $hShipment (@$aShippingLabels) {

        *DE_EPAGES::GLS::API::RemoteGLS::serviceCall =
            *DE_EPAGES::GLS::t::MockupResponses::ServerPetition;

        my $hResult = $RemoteTransaction->ServiceCreateLabel($hShipment);

        cmp_ok($hResult->{'errorcode'} ,'==', 'E000',
        'Response on shipping label ');
    }
    return;
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $System = LoadRootObject();
    $System->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] flags

options:
    -location    {call public location web service instead of smoking it}
    -shipment    {call shipment + shipping label PDF document generation web service instead of smoking it}
    -pickup      {call pickup web service instead of smoking it}
    -expedition  {call expedition PDF document generation web service instead of smoking it}
    -newuser     {call new user service instead of smoking it}

flags:
    -help       show the command line options

Examples:
    perl $0
    perl $0 -pickup -expedition

END_USAGE
    exit 2;
}


sub Main {
    my ( $Help );

    GetOptions (
        'help'          => \$Help,
    );

    usage() if $Help;

    run_tests();
    done_testing;
}


RunScript( 'Sub' => \&Main );

1;